

//new CoVaR Quantile Estimation program


capture program drop covar
program covar
syntax ,sys(integer) id(varlist) tvar(varlist) svarlist(varlist) quant(real)
sort `id' `tvar'
drop if `id' == .
tempvar id2
egen `id2' = group(`id') if `id' ~= `sys'
replace `id2' = 0 if `id' == `sys'
sum `id2' if `id2' ~= 0
local idmin = r(min)
local idmax = r(max)
sort `id2' `tvar'

preserve
display "firm is `firm'"
display "sys is `sys'"
display "idvar is `id'"
display "asofQtr is `asofQtr'"
display "statevars are `svarlist'"

keep `id2' `tvar' ret `svarlist'

//reshape data
reshape wide ret, i(`tvar') j(`id2')
save temp, replace
sort `tvar'

//estimate covar for each firm in this loop
quietly forvalues firm = `idmin'(1)`idmax' {
   noisily display "Estimating CoVaR for `firm'"
   //conditional CoVaR, get betas
   qreg ret0 ret`firm' `svarlist', q(`quant')
   gen cbeta = _b[ret`firm'] if e(sample)

   //conditional VaRs, get qtile and median VaR  
   qreg ret`firm' `svarlist', q(`quant') 
   predict cVaR if e(sample)
   qreg ret`firm' `svarlist', q(50)
   predict cmed if e(sample)

   //calculate cCoVaR
   gen cCoVaR`firm' = cbeta*(cVaR-cmed)
   drop ret`firm' cVaR cmed cbeta
}
drop ret0 `svarlist'
//now we should just have `tvar' and all the firm cCoVaRs
reshape long cCoVaR, i(`tvar') j(`id2')
drop if cCoVaR == .
sort `id2' `tvar'
tempfile estimates
save `estimates', replace

//merge back in with input data
restore
merge 1:1 `id2' `tvar' using `estimates'
drop _merge

end


